use rocket::{get, Responder, State};
use rocket::serde::json::Json;
use serde::Serialize;

use crate::ctx;
use crate::ctx::Ctx;

#[derive(Serialize)]
pub struct WebFinger {
  subject: String,
  aliases: Vec<String>,
  links: Vec<Link>,
}

#[derive(Serialize)]
pub struct Link {
  rel: String,
  #[serde(rename(serialize = "type"))]
  type_: String,
  href: String,
}

#[derive(Responder)]
pub enum Response {
  #[response(status = 200, content_type = "json")]
  Found(Json<WebFinger>),

  #[response(status = 404)]
  Missing(String),

  #[response(status = 400)]
  ParseError(String),

  #[response(status = 500)]
  InternalError(String),
}

#[get("/?<resource>")]
pub fn main(ctx: &State<Ctx>, resource: &str) -> Response {
  match parse_resource(&resource) {
    Ok(("acct", idstr)) | Ok(("", idstr)) => match ctx::user::Id::parse(idstr) {
      Ok(id) => match ctx.db.exists(&id) {
        Ok(true) => Response::Found(Json(WebFinger {
          subject: resource.to_owned(),
          aliases: vec![resource.to_owned()],
          links: vec![
            Link {
              rel: "self".to_owned(),
              type_: "application/activity+json".to_owned(),
              href: format!("{}/{}", ctx.cfg.get_baseurl(), id.name),
            },
          ],
        })),

        Ok(false) => Response::Missing(format!("unknown resource: {}", resource)),
        Err(msg) => Response::InternalError(format!("error: {}", msg))
      }
      Err(msg) => Response::ParseError(format!("error: {}", msg)),
    },
    Ok((kind, _)) => Response::ParseError(format!("error: unknown kind, {}", kind)),
    Err(msg) => Response::ParseError(format!("error: {}", msg)),
  }
}

fn parse_resource(v: &str) -> Result<(&str, &str), String> {
  let mut toks = v.split(':');
  match (toks.next(), toks.next(), toks.next()) {
    (Some(id),   None,     None) => Ok(("",   id)),
    (Some(kind), Some(id), None) => Ok((kind, id)),
    _ => Err(format!("invalid resource specifier: {}", v)),
  }
}
